# myfacerec/plugins/sample_plugin.py

from ..embedders import FaceEmbedder
from PIL import Image
import numpy as np
import torch

class SampleEmbedder(FaceEmbedder):
    def __init__(self):
        # Initialize your custom embedder here
        pass

    def embed_faces_batch(self, image: Image.Image, boxes: List[Tuple[int, int, int, int]]):
        # Implement custom embedding logic
        # For demonstration, return a dummy embedding
        if not boxes:
            return np.array([])
        return np.array([np.random.rand(512).astype(np.float32) for _ in boxes])



# myfacerec/cli.py

import argparse
import sys
from PIL import Image
import shutil
import os
from concurrent.futures import ThreadPoolExecutor, as_completed

from .config import Config
from .facial_recognition import FacialRecognition
from .hooks import Hooks
from ultralytics import YOLO
import torch
from facenet_pytorch import InceptionResnetV1
from .combined_model import CombinedFacialRecognitionModel  # Import the combined model
import numpy as np

def main():
    parser = argparse.ArgumentParser(
        prog="rolo-rec",
        description="Rolo-Rec: Modular and Extensible Facial Recognition Tool."
    )
    subparsers = parser.add_subparsers(dest="command")

    # -------------------------------------------------------------------------
    # Register command
    # -------------------------------------------------------------------------
    reg_parser = subparsers.add_parser("register", help="Register a user.")
    reg_parser.add_argument("--user", required=True, help="User ID (unique).")
    reg_parser.add_argument("--images", nargs="+", required=True, help="Paths to one or more face images.")
    reg_parser.add_argument("--conf", type=float, default=0.5, help="YOLO confidence threshold.")
    reg_parser.add_argument("--model-path", default=None,
                            help="Path or URL to custom YOLO model (.pt). If not set, default model is used.")
    reg_parser.add_argument("--detector", default=None,
                            help="Name of the detector plugin to use.")
    reg_parser.add_argument("--embedder", default=None,
                            help="Name of the embedder plugin to use.")
    reg_parser.add_argument("--parallelism", type=int, default=4,
                            help="Number of parallel threads for processing.")
    reg_parser.add_argument("--combined-model-path", default=None,
                            help="Path to a combined YOLO and Facenet .pt model.")

    # -------------------------------------------------------------------------
    # Identify command
    # -------------------------------------------------------------------------
    id_parser = subparsers.add_parser("identify", help="Identify faces in an image.")
    id_parser.add_argument("--image", required=True, help="Path to input image.")
    id_parser.add_argument("--threshold", type=float, default=0.6, help="Recognition threshold.")
    id_parser.add_argument("--conf", type=float, default=0.5, help="YOLO confidence threshold.")
    id_parser.add_argument("--model-path", default=None,
                           help="Path or URL to custom YOLO model (.pt). If not set, default model is used.")
    id_parser.add_argument("--detector", default=None,
                           help="Name of the detector plugin to use.")
    id_parser.add_argument("--embedder", default=None,
                           help="Name of the embedder plugin to use.")
    id_parser.add_argument("--combined-model-path", default=None,
                           help="Path to a combined YOLO and Facenet .pt model.")

    # -------------------------------------------------------------------------
    # Export data
    # -------------------------------------------------------------------------
    export_parser = subparsers.add_parser("export-data", help="Export the face data (JSON) to a specified file.")
    export_parser.add_argument("--output", required=True, help="Path to save the exported data.")

    # -------------------------------------------------------------------------
    # Import data
    # -------------------------------------------------------------------------
    import_parser = subparsers.add_parser("import-data", help="Import face data from a JSON file.")
    import_parser.add_argument("--file", required=True, help="Path to the JSON file to import.")

    # -------------------------------------------------------------------------
    # List users
    # -------------------------------------------------------------------------
    list_parser = subparsers.add_parser("list-users", help="List all registered users.")
    list_parser.add_argument("--combined-model-path", default=None,
                             help="Path to a combined YOLO and Facenet .pt model.")

    # -------------------------------------------------------------------------
    # Delete user
    # -------------------------------------------------------------------------
    delete_parser = subparsers.add_parser("delete-user", help="Delete a registered user.")
    delete_parser.add_argument("--user", required=True, help="User ID to delete.")
    delete_parser.add_argument("--combined-model-path", default=None,
                               help="Path to a combined YOLO and Facenet .pt model.")

    # -------------------------------------------------------------------------
    # Export model
    # -------------------------------------------------------------------------
    export_model_parser = subparsers.add_parser("export-model", help="Export the current model and user data as a .pt file.")
    export_model_parser.add_argument("--export-path", required=True, help="Path to save the exported .pt model.")
    export_model_parser.add_argument("--model-path", default=None,
                                     help="Path or URL to custom YOLO model (.pt). If not set, default model is used.")
    export_model_parser.add_argument("--detector", default=None,
                                     help="Name of the detector plugin to use.")
    export_model_parser.add_argument("--embedder", default=None,
                                     help="Name of the embedder plugin to use.")
    export_model_parser.add_argument("--combined-model-path", default=None,
                                     help="Path to a combined YOLO and Facenet .pt model.")

    # -------------------------------------------------------------------------
    # Import model
    # -------------------------------------------------------------------------
    import_model_parser = subparsers.add_parser("import-model", help="Import a combined model from a .pt file.")
    import_model_parser.add_argument("--import-path", required=True, help="Path to the .pt model to import.")
    import_model_parser.add_argument("--model-path", default=None,
                                     help="Path or URL to custom YOLO model (.pt). If not set, default model is used.")
    import_model_parser.add_argument("--detector", default=None,
                                     help="Name of the detector plugin to use.")
    import_model_parser.add_argument("--embedder", default=None,
                                     help="Name of the embedder plugin to use.")

    args = parser.parse_args()

    if not args.command:
        parser.print_help()
        sys.exit(1)

    # -------------------------------------------------------------------------
    # Build the Config / FacialRecognition objects where needed
    # -------------------------------------------------------------------------
    # For commands that don't require FacialRecognition (e.g., export-model, import-model), handle separately.

    if args.command in ["register", "identify", "list-users", "delete-user"]:
        config = Config(
            conf_threshold=args.conf if hasattr(args, 'conf') else 0.5,
            yolo_model_path=args.model_path,
            detector_plugin=args.detector,
            embedder_plugin=args.embedder
        )
        # Default to 'face.pt' if combined_model_path not provided
        combined_model_path = args.combined_model_path or "models/face.pt"
        fr = FacialRecognition(
            config,
            combined_model_path=combined_model_path
        )

    # -------------------------------------------------------------------------
    # Handle each command
    # -------------------------------------------------------------------------
    if args.command == "register":
        # Batch processing with parallelism
        def process_image(image_path):
            try:
                pil_image = Image.open(image_path).convert("RGB")
                return pil_image
            except Exception as e:
                print(f"[Error] Failed to open image {image_path}: {e}")
                return None

        pil_images = []
        with ThreadPoolExecutor(max_workers=args.parallelism) as executor:
            future_to_image = {executor.submit(process_image, img): img for img in args.images}
            for future in as_completed(future_to_image):
                img = future_to_image[future]
                result = future.result()
                if result:
                    pil_images.append(result)

        if not pil_images:
            print("[Error] No valid images to register.")
            sys.exit(1)

        msg = fr.register_user(args.user, pil_images)
        print(msg)

    elif args.command == "identify":
        try:
            pil_image = Image.open(args.image).convert("RGB")
        except Exception as e:
            print(f"[Error] Failed to open image {args.image}: {e}")
            sys.exit(1)

        results = fr.identify_user(pil_image, threshold=args.threshold)
        for i, r in enumerate(results):
            print(f"Face {i+1}: box={r['box']}, user_id='{r['user_id']}', similarity={r['similarity']:.2f}")

    elif args.command == "export-data":
        data_file = Config().user_data_path  # default user_faces.json
        if not os.path.exists(data_file):
            print("[Export] No user data found to export.")
            sys.exit(0)
        try:
            shutil.copyfile(data_file, args.output)
            print(f"[Export] Data exported to {args.output}")
        except Exception as e:
            print(f"[Export] Failed: {e}")

    elif args.command == "import-data":
        data_file = Config().user_data_path  # default user_faces.json
        if not os.path.exists(args.file):
            print(f"[Import] File '{args.file}' does not exist.")
            sys.exit(0)
        try:
            shutil.copyfile(args.file, data_file)
            print(f"[Import] Data imported from {args.file}")
        except Exception as e:
            print(f"[Import] Failed: {e}")

    elif args.command == "list-users":
        users = fr.list_users()
        if users:
            print("Registered Users:")
            for user in users:
                if hasattr(fr, 'combined_model'):
                    count = len(fr.combined_model.user_embeddings[user])
                else:
                    count = len(fr.user_data[user])
                print(f"- {user} ({count} embeddings)")
        else:
            print("No users registered.")

    elif args.command == "delete-user":
        success = fr.delete_user(args.user)
        if success:
            print(f"[Delete] User '{args.user}' has been deleted.")
        else:
            print(f"[Delete] User '{args.user}' does not exist.")

    elif args.command == "export-model":
        config = Config(
            conf_threshold=0.5,
            yolo_model_path=args.model_path,
            detector_plugin=args.detector,
            embedder_plugin=args.embedder
        )
        combined_model_path = args.combined_model_path or "models/face.pt"
        fr = FacialRecognition(
            config,
            combined_model_path=combined_model_path
        )
        message = fr.export_model(args.export_path)
        print(message)

    elif args.command == "import-model":
        if not os.path.exists(args.import_path):
            print(f"[Import Model] File '{args.import_path}' does not exist.")
            sys.exit(1)
        config = Config(
            conf_threshold=0.5,
            yolo_model_path=args.model_path,
            detector_plugin=args.detector,
            embedder_plugin=args.embedder
        )
        fr = FacialRecognition.import_model(args.import_path, config)
        print(f"[Import Model] Combined model loaded from {args.import_path}")

    else:
        parser.print_help()
        sys.exit(1)

if __name__ == "__main__":
    main()



# myfacerec/hooks.py

from typing import Callable, List

class Hooks:
    def __init__(self):
        self.before_detect: List[Callable] = []
        self.after_detect: List[Callable] = []
        self.before_embed: List[Callable] = []
        self.after_embed: List[Callable] = []

    def register_before_detect(self, func: Callable):
        self.before_detect.append(func)

    def register_after_detect(self, func: Callable):
        self.after_detect.append(func)

    def register_before_embed(self, func: Callable):
        self.before_embed.append(func)

    def register_after_embed(self, func: Callable):
        self.after_embed.append(func)

    def execute_before_detect(self, image):
        for func in self.before_detect:
            image = func(image)
        return image

    def execute_after_detect(self, boxes):
        for func in self.after_detect:
            func(boxes)

    def execute_before_embed(self, image, boxes):
        for func in self.before_embed:
            func(image, boxes)

    def execute_after_embed(self, embeddings):
        for func in self.after_embed:
            func(embeddings)



# myfacerec/embedders.py

import abc
import numpy as np
import torch
from PIL import Image
from typing import List, Tuple  # <-- Add this line

class FaceEmbedder(abc.ABC):
    """Abstract base class for face embedding."""
    @abc.abstractmethod
    def embed_faces_batch(self, image, boxes: List[Tuple[int, int, int, int]]):
        pass


class FacenetEmbedder(FaceEmbedder):
    def __init__(self, model, device="cpu", alignment_fn=None):
        self.model = model
        self.device = device
        self.alignment_fn = alignment_fn
        self.model.eval()

    def embed_faces_batch(self, image: Image.Image, boxes: List[Tuple[int, int, int, int]]):
        if not boxes:
            return np.array([])

        face_tensors = []
        for (x1, y1, x2, y2) in boxes:
            face = image.crop((x1, y1, x2, y2)).resize((160, 160))

            if self.alignment_fn:
                face = self.alignment_fn(face)

            face_np = np.array(face).astype(np.float32) / 255.0
            face_np = (face_np - 0.5) / 0.5
            face_tensor = torch.from_numpy(face_np).permute(2, 0, 1).float()
            face_tensors.append(face_tensor)

        if not face_tensors:
            return np.array([])

        batch = torch.stack(face_tensors).to(self.device)
        with torch.no_grad():
            embeddings = self.model(batch).cpu().numpy()
        return embeddings



# myfacerec/combined_model.py

import torch
import torch.nn as nn
from ultralytics import YOLO
from facenet_pytorch import InceptionResnetV1
from PIL import Image
import numpy as np
import os
from typing import Optional, List, Tuple
import pkg_resources

class CombinedFacialRecognitionModel(nn.Module):
    def __init__(self, device: str = 'cpu'):
        super(CombinedFacialRecognitionModel, self).__init__()
        self.device = device

        # Initialize YOLO model without loading weights yet
        self.yolo = YOLO()
        self.yolo.to(self.device)

        # Initialize Facenet model
        self.facenet = InceptionResnetV1(pretrained='vggface2').eval().to(self.device)

        # User embeddings: Dict[str, List[np.ndarray]]
        self.user_embeddings = {}

    def forward(self, image: Image.Image) -> List[Tuple[Tuple[int, int, int, int], np.ndarray]]:
        """
        Perform face detection and embedding extraction.

        Args:
            image (PIL.Image): Input image.

        Returns:
            List of tuples: Each tuple contains bounding box coordinates and the corresponding embedding.
        """
        # Ensure image is a PIL Image
        if isinstance(image, torch.Tensor):
            image = image.permute(1, 2, 0).cpu().numpy()
            image = Image.fromarray((image * 255).astype('uint8'))

        boxes = []
        # Detect faces using YOLO
        detections = self.yolo(image)
        for result in detections:
            for box in result.boxes:
                conf = box.conf.item()
                cls = int(box.cls.item())
                if conf >= 0.5 and cls == 0:  # Assuming class 0 is 'face'
                    x1, y1, x2, y2 = box.xyxy[0].tolist()
                    boxes.append((int(x1), int(y1), int(x2), int(y2)))

        embeddings = []
        for box in boxes:
            face = image.crop(box).resize((160, 160))
            face_np = np.array(face).astype(np.float32) / 255.0
            face_np = (face_np - 0.5) / 0.5
            face_tensor = torch.from_numpy(face_np).permute(2, 0, 1).unsqueeze(0).to(self.device)
            with torch.no_grad():
                emb = self.facenet(face_tensor).cpu().numpy()[0]
            embeddings.append(emb)

        return list(zip(boxes, embeddings))

    def save_model(self, save_path: str):
        """
        Save the combined model and user embeddings to a .pt file.

        Args:
            save_path (str): Path to save the .pt model.
        """
        # Prepare state dictionary
        state = {
            'yolo_state_dict': self.yolo.model.state_dict(),
            'facenet_state_dict': self.facenet.state_dict(),
            'user_embeddings': self.user_embeddings,
            'device': self.device,
            'yolo_model_path': self.yolo.model_path if hasattr(self.yolo, 'model_path') else None
        }
        torch.save(state, save_path)

    @classmethod
    def load_model(cls, load_path: str, device: str = 'cpu'):
        """
        Load the combined model and user embeddings from a .pt file.

        Args:
            load_path (str): Path to the .pt model.
            device (str): Device to load the model on.

        Returns:
            CombinedFacialRecognitionModel: Loaded model.
        """
        state = torch.load(load_path, map_location=device)
        model = cls(device=device)

        # Load YOLO state_dict
        if state['yolo_state_dict']:
            model.yolo.model.load_state_dict(state['yolo_state_dict'])

        # Load Facenet state_dict
        model.facenet.load_state_dict(state['facenet_state_dict'])

        # Load user embeddings
        model.user_embeddings = state['user_embeddings']

        model.to(device)
        model.eval()
        return model



import os
import requests
import logging
import numpy as np
from sklearn.metrics.pairwise import cosine_similarity
from typing import List, Optional, Tuple, Dict, Any  # Added Dict and Any

from PIL import Image

from .config import Config, logger
from .detectors import YOLOFaceDetector, FaceDetector
from .embedders import FacenetEmbedder, FaceEmbedder
from .data_store import JSONUserDataStore, UserDataStore
from .hooks import Hooks
from .plugins.base import PluginManager
from .combined_model import CombinedFacialRecognitionModel

import torch
from ultralytics import YOLO
from facenet_pytorch import InceptionResnetV1


class FacialRecognition:
    """
    Main orchestrator that uses:
      - a FaceDetector plugin or a combined model
      - a FaceEmbedder plugin (if not using combined model)
      - a UserDataStore plugin
      - config hooks
    """

    def __init__(
        self,
        config: Config,
        detector: Optional[FaceDetector] = None,
        embedder: Optional[FaceEmbedder] = None,
        data_store: Optional[UserDataStore] = None,
        combined_model_path: Optional[str] = None  # New parameter for combined model
    ):
        self.config = config
        self.logger = logging.getLogger(__name__)
        self.hooks = Hooks()
        self.plugin_manager = PluginManager()

        # Initialize combined model if path is provided
        if combined_model_path:
            if not os.path.exists(combined_model_path):
                raise FileNotFoundError(f"Combined model file not found: {combined_model_path}")
            try:
                self.combined_model = CombinedFacialRecognitionModel.load_model(
                    combined_model_path, device=config.device
                )
                self.logger.info("Initialized with combined YOLO and Facenet model.")
            except Exception as e:
                self.logger.error(f"Failed to load combined model: {e}")
                raise
        else:
            self._initialize_separate_components(detector, embedder, data_store)

    def _initialize_separate_components(
        self,
        detector: Optional[FaceDetector],
        embedder: Optional[FaceEmbedder],
        data_store: Optional[UserDataStore]
    ):
        """Initialize individual components when combined model is not used."""
        if self.config.detector_plugin:
            detector = self.plugin_manager.load_detector(self.config.detector_plugin)
        if self.config.embedder_plugin:
            embedder = self.plugin_manager.load_embedder(self.config.embedder_plugin)

        if detector is None:
            detector = self._initialize_detector()
        self.detector = detector

        if embedder is None:
            embedder = self._initialize_embedder()
        self.embedder = embedder

        if data_store is None:
            data_store = JSONUserDataStore(self.config.user_data_path)
        self.data_store = data_store

        self.user_data = self.data_store.load_user_data()
        self.logger.info("Initialized with %d users in data store.", len(self.user_data))

    def _initialize_detector(self) -> YOLOFaceDetector:
        """Initialize YOLO detector."""
        path = self.config.yolo_model_path or self.config.default_model_url
        if path.startswith("http"):
            path = self._download_model(path)
        yolo_model = YOLO(path)
        yolo_model.to(self.config.device)
        return YOLOFaceDetector(yolo_model, conf_threshold=self.config.conf_threshold)

    def _initialize_embedder(self) -> FacenetEmbedder:
        """Initialize Facenet embedder."""
        fn_model = InceptionResnetV1(pretrained='vggface2').eval().to(self.config.device)
        return FacenetEmbedder(fn_model, device=self.config.device, alignment_fn=self.config.alignment_fn)

    def _download_model(self, url: str) -> str:
        """Download and cache model if not already present."""
        base_dir = os.path.join(self.config.cache_dir, "models")
        os.makedirs(base_dir, exist_ok=True)
        model_path = os.path.join(base_dir, os.path.basename(url))

        if not os.path.exists(model_path):
            self.logger.info(f"Downloading model from {url}")
            response = requests.get(url, stream=True)
            response.raise_for_status()
            with open(model_path, 'wb') as file:
                for chunk in response.iter_content(chunk_size=8192):
                    file.write(chunk)
            self.logger.info(f"Model saved to {model_path}")
        return model_path

    def detect_faces(self, image: Image.Image) -> Tuple[List[Tuple[int, int, int, int]], Optional[List[np.ndarray]]]:
        if hasattr(self, 'combined_model'):
            results = self.combined_model(image)
            boxes, embeddings = zip(*results) if results else ([], [])
            return list(boxes), list(embeddings)
        else:
            if self.hooks.before_detect:
                image = self.hooks.execute_before_detect(image)
            boxes = self.detector.detect_faces(image)
            if self.hooks.after_detect:
                self.hooks.execute_after_detect(boxes)
            return boxes, None

    def embed_faces_batch(self, image: Image.Image, boxes: List[Tuple[int, int, int, int]]) -> List[np.ndarray]:
        if hasattr(self, 'combined_model'):
            _, embeddings = self.detect_faces(image)
            return embeddings
        else:
            if self.hooks.before_embed:
                self.hooks.execute_before_embed(image, boxes)
            embeddings = self.embedder.embed_faces_batch(image, boxes)
            if self.hooks.after_embed:
                self.hooks.execute_after_embed(embeddings)
            return embeddings

    def register_user(self, user_id: str, images: List[Image.Image]) -> str:
        collected_embeddings = []
        for img in images:
            boxes, embeddings = self.detect_faces(img)
            if len(boxes) == 1 and embeddings:
                collected_embeddings.append(embeddings[0])
        if not collected_embeddings:
            return f"[Registration] No valid single-face images found for '{user_id}'."
        if hasattr(self, 'combined_model'):
            if user_id not in self.combined_model.user_embeddings:
                self.combined_model.user_embeddings[user_id] = []
            self.combined_model.user_embeddings[user_id].extend(collected_embeddings)
        else:
            if user_id not in self.user_data:
                self.user_data[user_id] = []
            self.user_data[user_id].extend(collected_embeddings)
            self.data_store.save_user_data(self.user_data)
        return f"[Registration] User '{user_id}' registered with {len(collected_embeddings)} images."

    def identify_user(self, image: Image.Image, threshold=0.6) -> List[Dict[str, Any]]:
        boxes, embeddings = self.detect_faces(image)
        if not boxes:
            return []
        results = []
        for i, emb in enumerate(embeddings):
            best_match = None
            best_sim = 0.0
            user_embeddings = self.combined_model.user_embeddings if hasattr(self, 'combined_model') else self.user_data
            for user_id, emb_list in user_embeddings.items():
                valid_embs = [e for e in emb_list if e.shape == emb.shape]
                if not valid_embs:
                    continue
                sims = cosine_similarity([emb], valid_embs)
                max_sim = np.max(sims)
                if max_sim > best_sim:
                    best_sim = max_sim
                    best_match = user_id
            if best_sim >= threshold:
                results.append({'box': boxes[i], 'user_id': best_match, 'similarity': float(best_sim)})
            else:
                results.append({'box': boxes[i], 'user_id': 'Unknown', 'similarity': float(best_sim)})
        return results

    def export_model(self, export_path: str) -> str:
        if hasattr(self, 'combined_model'):
            self.combined_model.save_model(export_path)
            return f"Combined model saved to {export_path}"
        else:
            state = {
                'yolo_state_dict': self.detector.model.state_dict(),
                'facenet_state_dict': self.embedder.model.state_dict(),
                'user_embeddings': self.user_data,
                'device': self.config.device
            }
            torch.save(state, export_path)
            return f"Model and user data saved to {export_path}"

    @classmethod
    def import_model(cls, import_path: str, config: Config) -> "FacialRecognition":
        combined_model = CombinedFacialRecognitionModel.load_model(import_path, device=config.device)
        return cls(config, combined_model_path=import_path)



# myfacerec/detectors.py

import abc
from typing import List, Tuple

class FaceDetector(abc.ABC):
    """Abstract base class for face detectors."""
    @abc.abstractmethod
    def detect_faces(self, image) -> List[Tuple[int, int, int, int]]:
        pass


class YOLOFaceDetector(FaceDetector):
    def __init__(self, model, conf_threshold=0.5):
        self.model = model
        self.conf_threshold = conf_threshold

    def detect_faces(self, image):
        results = self.model(image)
        boxes = []
        for result in results:
            for box in result.boxes:
                conf = box.conf.item()
                cls = int(box.cls.item())
                if conf >= self.conf_threshold and cls == 0:
                    x1, y1, x2, y2 = box.xyxy[0].tolist()
                    boxes.append((int(x1), int(y1), int(x2), int(y2)))
        return boxes



# setup.py

from setuptools import setup, find_packages

setup(
    name="rolo-rec",  # Renamed to "rolo-rec" for PyPI
    version="0.2.0",
    description="Future-proof Facial Recognition with YOLO + Facenet, modular detectors/embedders, hooks, CLI, etc.",
    author="wuhp",
    packages=find_packages(),
    install_requires=[
        "requests",
        "numpy",
        "torch",
        "Pillow",
        "scikit-learn",
        "ultralytics",
        "facenet-pytorch",
    ],
    python_requires=">=3.7",
    entry_points={
        "console_scripts": [
            # This means users can now run `rolo-rec` from the command line
            "rolo-rec=myfacerec.cli:main", 
        ],
    },
)



# myfacerec/__init__.py

from .facial_recognition import FacialRecognition
from .config import Config
from .detectors import YOLOFaceDetector
from .embedders import FacenetEmbedder
from .data_store import JSONUserDataStore
from .hooks import Hooks
from .combined_model import CombinedFacialRecognitionModel

__all__ = [
    "FacialRecognition",
    "Config",
    "YOLOFaceDetector",
    "FacenetEmbedder",
    "JSONUserDataStore",
    "Hooks",
    "CombinedFacialRecognitionModel",
]



# myfacerec/data_store.py

import abc
import os
import json
import numpy as np

class UserDataStore(abc.ABC):
    """Abstract interface for storing/retrieving user embeddings."""
    @abc.abstractmethod
    def load_user_data(self):
        pass

    @abc.abstractmethod
    def save_user_data(self, user_data):
        pass


class JSONUserDataStore(UserDataStore):
    def __init__(self, path="user_faces.json"):
        self.path = path

    def load_user_data(self):
        if not os.path.exists(self.path):
            with open(self.path, 'w') as f:
                json.dump({}, f)

        with open(self.path, 'r') as f:
            data = json.load(f)

        # Convert lists -> np.array
        for user_id in data:
            data[user_id] = [np.array(e) for e in data[user_id]]
        return data

    def save_user_data(self, user_data):
        serializable_data = {
            user_id: [emb.tolist() for emb in emb_list]
            for user_id, emb_list in user_data.items()
        }
        with open(self.path, 'w') as f:
            json.dump(serializable_data, f)



# myfacerec/plugins/base.py

import pkg_resources
import logging
from typing import Type

from ..detectors import FaceDetector
from ..embedders import FaceEmbedder
from ..data_store import UserDataStore

logger = logging.getLogger(__name__)

class PluginManager:
    """
    Manages loading of detector, embedder, and data store plugins.
    Plugins must be registered via entry points in setup.py.
    """

    def load_detector(self, name: str) -> FaceDetector:
        try:
            entry_point = pkg_resources.get_entry_map('rolo-rec')['rolo_rec.detectors'][name]
            detector_class = entry_point.load()
            logger.info(f"Loaded detector plugin: {name}")
            return detector_class()
        except KeyError:
            logger.error(f"Detector plugin '{name}' not found.")
            raise
        except Exception as e:
            logger.error(f"Failed to load detector plugin '{name}': {e}")
            raise

    def load_embedder(self, name: str) -> FaceEmbedder:
        try:
            entry_point = pkg_resources.get_entry_map('rolo-rec')['rolo_rec.embedders'][name]
            embedder_class = entry_point.load()
            logger.info(f"Loaded embedder plugin: {name}")
            return embedder_class()
        except KeyError:
            logger.error(f"Embedder plugin '{name}' not found.")
            raise
        except Exception as e:
            logger.error(f"Failed to load embedder plugin '{name}': {e}")
            raise

    def load_data_store(self, name: str) -> UserDataStore:
        try:
            entry_point = pkg_resources.get_entry_map('rolo-rec')['rolo_rec.data_stores'][name]
            data_store_class = entry_point.load()
            logger.info(f"Loaded data store plugin: {name}")
            return data_store_class()
        except KeyError:
            logger.error(f"Data store plugin '{name}' not found.")
            raise
        except Exception as e:
            logger.error(f"Failed to load data store plugin '{name}': {e}")
            raise



# setup.py

from setuptools import setup, find_packages

setup(
    name="rolo-rec",
    version="0.4.2",
    description="Future-proof Facial Recognition with YOLO + Facenet, modular detectors/embedders, hooks, CLI, etc.",
    author="wuhp",
    packages=find_packages(),
    include_package_data=True,  # Ensures package data from MANIFEST.in is included
    install_requires=[
        "requests",
        "numpy",
        "torch",
        "Pillow",
        "scikit-learn",
        "ultralytics",
        "facenet-pytorch",
        "torchvision<0.18.0,>=0.17.0",
        "tqdm<5.0.0,>=4.0.0",
    ],
    extras_require={
        "dev": [
            "pytest",
            "pluggy<2,>=1.5",
            "iniconfig",
        ],
    },
    python_requires=">=3.9",
    entry_points={
        "console_scripts": [
            "rolo-rec=myfacerec.cli:main",
        ],
        "rolo_rec.detectors": [
            # Add custom detectors here or in plugins
        ],
        "rolo_rec.embedders": [
            # Add custom embedders here or in plugins
        ],
        "rolo_rec.data_stores": [
            # Add custom data stores here or in plugins
        ],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
)



# myfacerec/config.py

import logging
import os

class Config:
    """
    Central configuration object.
    """
    def __init__(
        self,
        yolo_model_path=None,
        default_model_url="https://raw.githubusercontent.com/wuhplaptop/facerec/main/face.pt",
        conf_threshold=0.5,
        device=None,
        user_data_path="user_faces.json",
        alignment_fn=None,     # Optional face alignment function
        before_detect=None,
        after_detect=None,
        before_embed=None,
        after_embed=None,
        detector_plugin=None,
        embedder_plugin=None,
        cache_dir=None
    ):
        self.yolo_model_path = yolo_model_path or self._default_model_path()
        self.default_model_url = default_model_url
        self.conf_threshold = conf_threshold
        self.device = device or self._auto_device()
        self.user_data_path = user_data_path

        # Optional hooks
        self.alignment_fn = alignment_fn
        self.before_detect = before_detect
        self.after_detect = after_detect
        self.before_embed = before_embed
        self.after_embed = after_embed

        # Plugins
        self.detector_plugin = detector_plugin
        self.embedder_plugin = embedder_plugin

        # Caching
        self.cache_dir = cache_dir or os.path.expanduser("~/.myfacerec/cache")

    def _auto_device(self):
        import torch
        return "cuda" if torch.cuda.is_available() else "cpu"

    def _default_model_path(self):
        """
        Returns the default model path, which is 'models/face.pt' within the package.
        """
        import pkg_resources
        return pkg_resources.resource_filename(__name__, 'models/face.pt')


# Basic logger setup (global)
logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s [%(levelname)s] %(name)s: %(message)s"
)
logger = logging.getLogger(__name__)
